@@ -1,3 +1,4 @@
+use crate::cli::Cli;
 use crate::config::Config;
 use crate::config::Mode;
 use crate::explorer;
@@ -1084,6 +1085,7 @@ pub enum ExternalMsg {
     ResetInputBuffer,
 
     /// Switch input mode.
+    /// It clears the input buffer.
     ///
     /// > **NOTE:** To be specific about which mode to switch to, use `SwitchModeBuiltin` or
     /// `SwitchModeCustom` instead.
@@ -1091,19 +1093,48 @@ pub enum ExternalMsg {
     /// **Example:** `SwitchMode: default`
     SwitchMode(String),
 
+    /// Switch input mode.
+    /// It keeps the input buffer.
+    ///
+    /// > **NOTE:** To be specific about which mode to switch to, use
+    /// `SwitchModeBuiltinKeepingInputBuffer` or
+    /// `SwitchModeCustomKeepingInputBuffer` instead.
+    ///
+    /// **Example:** `SwitchModeKeepingInputBuffer: default`
+    SwitchModeKeepingInputBuffer(String),
+
     /// Switch to a builtin mode.
+    /// It clears the input buffer.
     ///
     /// **Example:** `SwitchModeBuiltin: default`
     SwitchModeBuiltin(String),
 
+    /// Switch to a builtin mode.
+    /// It keeps the input buffer.
+    ///
+    /// **Example:** `SwitchModeBuiltinKeepingInputBuffer: default`
+    SwitchModeBuiltinKeepingInputBuffer(String),
+
     /// Switch to a custom mode.
+    /// It clears the input buffer.
     ///
     /// **Example:** `SwitchModeCustom: my_custom_mode`
     SwitchModeCustom(String),
 
+    /// Switch to a custom mode.
+    /// It keeps the input buffer.
+    ///
+    /// **Example:** `SwitchModeCustomKeepingInputBuffer: my_custom_mode`
+    SwitchModeCustomKeepingInputBuffer(String),
+
     /// Pop the last mode from the history and switch to it.
+    /// It clears the input buffer.
     PopMode,
 
+    /// Pop the last mode from the history and switch to it.
+    /// It keeps the input buffer.
+    PopModeKeepingInputBuffer,
+
     /// Switch layout.
     ///
     /// > **NOTE:** To be specific about which layout to switch to, use `SwitchLayoutBuiltin` or
@@ -1498,7 +1529,6 @@ impl History {
 #[derive(Debug, Clone, Serialize, Deserialize)]
 pub struct CallLuaArg {
     pub version: String,
-    pub config: Config,
     pub pwd: String,
     pub focused_node: Option<Node>,
     pub directory_buffer: Option<DirectoryBuffer>,
@@ -1744,9 +1774,19 @@ impl App {
                 ExternalMsg::RemoveInputBufferLastWord => self.remove_input_buffer_last_word(),
                 ExternalMsg::ResetInputBuffer => self.reset_input_buffer(),
                 ExternalMsg::SwitchMode(mode) => self.switch_mode(&mode),
+                ExternalMsg::SwitchModeKeepingInputBuffer(mode) => {
+                    self.switch_mode_keeping_input_buffer(&mode)
+                }
                 ExternalMsg::SwitchModeBuiltin(mode) => self.switch_mode_builtin(&mode),
+                ExternalMsg::SwitchModeBuiltinKeepingInputBuffer(mode) => {
+                    self.switch_mode_builtin_keeping_input_buffer(&mode)
+                }
                 ExternalMsg::SwitchModeCustom(mode) => self.switch_mode_custom(&mode),
+                ExternalMsg::SwitchModeCustomKeepingInputBuffer(mode) => {
+                    self.switch_mode_custom_keeping_input_buffer(&mode)
+                }
                 ExternalMsg::PopMode => self.pop_mode(),
+                ExternalMsg::PopModeKeepingInputBuffer => self.pop_mode_keeping_input_buffer(),
                 ExternalMsg::SwitchLayout(mode) => self.switch_layout(&mode),
                 ExternalMsg::SwitchLayoutBuiltin(mode) => self.switch_layout_builtin(&mode),
                 ExternalMsg::SwitchLayoutCustom(mode) => self.switch_layout_custom(&mode),
@@ -1829,15 +1869,10 @@ impl App {
             })
             .or_else(|| default.map(|a| a.messages().clone()))
             .unwrap_or_else(|| {
-                if self.config().general().disable_recover_mode() {
-                    vec![ExternalMsg::LogWarning("Key map not found.".into())]
+                if self.config().general().enable_recover_mode() {
+                    vec![ExternalMsg::SwitchModeBuiltin("recover".into())]
                 } else {
-                    vec![
-                        ExternalMsg::SwitchModeBuiltin("recover".into()),
-                        ExternalMsg::LogWarning(
-                            "Key map not found. Let's calm down, escape, and try again.".into(),
-                        ),
-                    ]
+                    vec![ExternalMsg::LogWarning("Key map not found.".into())]
                 }
             });
 
@@ -2211,18 +2246,30 @@ impl App {
         self
     }
 
-    fn pop_mode(mut self) -> Result<Self> {
+    fn pop_mode(self) -> Result<Self> {
+        self.pop_mode_keeping_input_buffer().map(|mut a| {
+            a.input_buffer = None;
+            a
+        })
+    }
+
+    fn pop_mode_keeping_input_buffer(mut self) -> Result<Self> {
         if let Some(mode) = self.last_modes.pop() {
-            self.input_buffer = None;
             self.mode = mode;
         };
         Ok(self)
     }
 
-    fn switch_mode(mut self, mode: &str) -> Result<Self> {
+    fn switch_mode(self, mode: &str) -> Result<Self> {
+        self.switch_mode_keeping_input_buffer(mode).map(|mut a| {
+            a.input_buffer = None;
+            a
+        })
+    }
+
+    fn switch_mode_keeping_input_buffer(mut self, mode: &str) -> Result<Self> {
         if let Some(mode) = self.config().modes().clone().get(mode) {
             self = self.push_mode();
-            self.input_buffer = None;
             self.mode = mode
                 .to_owned()
                 .sanitized(self.config().general().read_only());
@@ -2232,10 +2279,17 @@ impl App {
         }
     }
 
-    fn switch_mode_builtin(mut self, mode: &str) -> Result<Self> {
+    fn switch_mode_builtin(self, mode: &str) -> Result<Self> {
+        self.switch_mode_builtin_keeping_input_buffer(mode)
+            .map(|mut a| {
+                a.input_buffer = None;
+                a
+            })
+    }
+
+    fn switch_mode_builtin_keeping_input_buffer(mut self, mode: &str) -> Result<Self> {
         if let Some(mode) = self.config().modes().clone().get_builtin(mode) {
             self = self.push_mode();
-            self.input_buffer = None;
             self.mode = mode
                 .to_owned()
                 .sanitized(self.config().general().read_only());
@@ -2245,10 +2299,17 @@ impl App {
         }
     }
 
-    fn switch_mode_custom(mut self, mode: &str) -> Result<Self> {
+    fn switch_mode_custom(self, mode: &str) -> Result<Self> {
+        self.switch_mode_custom_keeping_input_buffer(mode)
+            .map(|mut a| {
+                a.input_buffer = None;
+                a
+            })
+    }
+
+    fn switch_mode_custom_keeping_input_buffer(mut self, mode: &str) -> Result<Self> {
         if let Some(mode) = self.config().modes().clone().get_custom(mode) {
             self = self.push_mode();
-            self.input_buffer = None;
             self.mode = mode
                 .to_owned()
                 .sanitized(self.config().general().read_only());
@@ -2342,10 +2403,8 @@ impl App {
     fn select(mut self) -> Result<Self> {
         if let Some(n) = self.focused_node().map(|n| n.to_owned()) {
             self.selection.insert(n);
-            Ok(self)
-        } else {
-            Ok(self)
         }
+        Ok(self)
     }
 
     fn select_path(mut self, path: String) -> Result<Self> {
@@ -2357,10 +2416,8 @@ impl App {
         let filename = path.file_name().map(|p| p.to_string_lossy().to_string());
         if let (Some(p), Some(n)) = (parent, filename) {
             self.selection.insert(Node::new(p, n));
-            Ok(self)
-        } else {
-            Ok(self)
         }
+        Ok(self)
     }
 
     fn select_all(mut self) -> Result<Self> {
@@ -2886,7 +2943,6 @@ impl App {
     pub fn to_lua_arg(&self) -> CallLuaArg {
         CallLuaArg {
             version: self.version.clone(),
-            config: self.config.clone(),
             pwd: self.pwd.clone(),
             focused_node: self.focused_node().cloned(),
             directory_buffer: self.directory_buffer().cloned(),
@@ -2903,6 +2959,12 @@ impl App {
     }
 }
 
-pub fn runner(path: Option<PathBuf>) -> Result<Runner> {
-    Runner::new(path)
+/// Create a new runner object passing the default arguments
+pub fn runner() -> Result<Runner> {
+    Runner::new()
+}
+
+/// Create a new runner object passing the given arguments
+pub fn from_cli(cli: Cli) -> Result<Runner> {
+    Runner::from_cli(cli)
 }
