@@ -1,6 +1,7 @@
 #![allow(clippy::too_many_arguments)]
 
 use crate::app;
+use crate::cli::Cli;
 use crate::event_reader;
 use crate::explorer;
 use crate::lua;
@@ -106,9 +107,16 @@ pub struct Runner {
 }
 
 impl Runner {
-    pub(crate) fn new(path: Option<PathBuf>) -> Result<Self> {
+    /// Create a new runner object passing the default arguments
+    pub fn new() -> Result<Self> {
+        Self::from_cli(Default::default())
+    }
+
+    /// Create a new runner object passing the given arguments
+    pub fn from_cli(cli: Cli) -> Result<Self> {
         let basedir = std::env::current_dir()?;
-        let mut pwd = path
+        let mut pwd = cli
+            .path
             .map(|p| if p.is_relative() { basedir.join(p) } else { p })
             .unwrap_or_else(|| basedir.clone());
         let mut focused_path = None;
@@ -121,33 +129,14 @@ impl Runner {
         Ok(Self {
             pwd,
             focused_path,
-            config_file: Default::default(),
-            extra_config_files: Default::default(),
-            on_load: Default::default(),
-            read_only: Default::default(),
+            config_file: cli.config,
+            extra_config_files: cli.extra_config,
+            on_load: cli.on_load,
+            read_only: cli.read_only,
         })
     }
 
-    pub fn with_on_load(mut self, on_load: Vec<app::ExternalMsg>) -> Self {
-        self.on_load = on_load;
-        self
-    }
-
-    pub fn with_config(mut self, config_file: Option<PathBuf>) -> Self {
-        self.config_file = config_file;
-        self
-    }
-
-    pub fn with_extra_config(mut self, config_files: Vec<PathBuf>) -> Self {
-        self.extra_config_files = config_files;
-        self
-    }
-
-    pub fn with_read_only(mut self, read_only: bool) -> Self {
-        self.read_only = read_only;
-        self
-    }
-
+    /// Run the instance
     pub fn run(self) -> Result<Option<String>> {
         // Why unsafe? See https://github.com/sayanarijit/xplr/issues/309
         let lua = unsafe { mlua::Lua::unsafe_new() };
@@ -158,6 +147,7 @@ impl Runner {
 
         let (tx_msg_in, rx_msg_in) = mpsc::channel();
         let (tx_event_reader, rx_event_reader) = mpsc::channel();
+        let (tx_loop_waiter, rx_loop_waiter) = mpsc::channel();
         let (tx_pwd_watcher, rx_pwd_watcher) = mpsc::channel();
 
         app = app.explore_pwd()?;
@@ -218,7 +208,7 @@ impl Runner {
         terminal.hide_cursor()?;
 
         // Threads
-        event_reader::keep_reading(tx_msg_in.clone(), rx_event_reader);
+        event_reader::keep_reading(tx_msg_in.clone(), rx_event_reader, tx_loop_waiter);
         pwd_watcher::keep_watching(app.pwd(), tx_msg_in.clone(), rx_pwd_watcher)?;
 
         // Enqueue on_load messages
@@ -231,55 +221,56 @@ impl Runner {
                 Ok(a) => {
                     app = a;
                     while let Some(msg) = app.pop_msg_out() {
+                        use app::MsgOut::*;
                         match msg {
                             // NOTE: Do not schedule critical tasks via tx_msg_in in this loop.
                             // Try handling them immediately.
-                            app::MsgOut::Enque(task) => {
+                            Enque(task) => {
                                 tx_msg_in.send(task)?;
                             }
 
-                            app::MsgOut::Quit => {
+                            Quit => {
                                 result = Ok(None);
                                 break 'outer;
                             }
 
-                            app::MsgOut::PrintPwdAndQuit => {
+                            PrintPwdAndQuit => {
                                 result = Ok(Some(format!("{}\n", app.pwd())));
                                 break 'outer;
                             }
 
-                            app::MsgOut::PrintFocusPathAndQuit => {
+                            PrintFocusPathAndQuit => {
                                 result = Ok(app
                                     .focused_node()
                                     .map(|n| format!("{}\n", n.absolute_path())));
                                 break 'outer;
                             }
 
-                            app::MsgOut::PrintSelectionAndQuit => {
+                            PrintSelectionAndQuit => {
                                 result = Ok(Some(app.selection_str()));
                                 break 'outer;
                             }
 
-                            app::MsgOut::PrintResultAndQuit => {
+                            PrintResultAndQuit => {
                                 result = Ok(Some(app.result_str()));
                                 break 'outer;
                             }
 
-                            app::MsgOut::PrintAppStateAndQuit => {
+                            PrintAppStateAndQuit => {
                                 let out = serde_yaml::to_string(&app)?;
                                 result = Ok(Some(out));
                                 break 'outer;
                             }
 
-                            app::MsgOut::Debug(path) => {
+                            Debug(path) => {
                                 fs::write(&path, serde_yaml::to_string(&app)?)?;
                             }
 
-                            app::MsgOut::ClearScreen => {
+                            ClearScreen => {
                                 terminal.clear()?;
                             }
 
-                            app::MsgOut::ExplorePwdAsync => {
+                            ExplorePwdAsync => {
                                 explorer::explore_async(
                                     app.explorer_config().clone(),
                                     app.pwd().into(),
@@ -290,7 +281,7 @@ impl Runner {
                                 tx_pwd_watcher.send(app.pwd().clone())?;
                             }
 
-                            app::MsgOut::ExploreParentsAsync => {
+                            ExploreParentsAsync => {
                                 explorer::explore_recursive_async(
                                     app.explorer_config().clone(),
                                     app.pwd().into(),
@@ -301,7 +292,7 @@ impl Runner {
                                 tx_pwd_watcher.send(app.pwd().clone())?;
                             }
 
-                            app::MsgOut::Refresh => {
+                            Refresh => {
                                 // $PWD watcher
                                 tx_pwd_watcher.send(app.pwd().clone())?;
                                 // UI
@@ -316,7 +307,7 @@ impl Runner {
                                 }
                             }
 
-                            app::MsgOut::EnableMouse => {
+                            EnableMouse => {
                                 if !mouse_enabled {
                                     match execute!(
                                         terminal.backend_mut(),
@@ -332,7 +323,7 @@ impl Runner {
                                 }
                             }
 
-                            app::MsgOut::ToggleMouse => {
+                            ToggleMouse => {
                                 let msg = if mouse_enabled {
                                     app::ExternalMsg::DisableMouse
                                 } else {
@@ -342,7 +333,7 @@ impl Runner {
                                     .handle_task(app::Task::new(app::MsgIn::External(msg), None))?;
                             }
 
-                            app::MsgOut::DisableMouse => {
+                            DisableMouse => {
                                 if mouse_enabled {
                                     match execute!(
                                         terminal.backend_mut(),
@@ -358,7 +349,7 @@ impl Runner {
                                 }
                             }
 
-                            app::MsgOut::StartFifo(path) => {
+                            StartFifo(path) => {
                                 fifo = match start_fifo(&path, &app.focused_node_str()) {
                                     Ok(file) => Some(file),
                                     Err(e) => {
@@ -368,14 +359,14 @@ impl Runner {
                                 }
                             }
 
-                            app::MsgOut::StopFifo => {
+                            StopFifo => {
                                 if let Some(file) = fifo {
                                     fifo = None;
                                     std::mem::drop(file);
                                 }
                             }
 
-                            app::MsgOut::ToggleFifo(path) => {
+                            ToggleFifo(path) => {
                                 if let Some(file) = fifo {
                                     fifo = None;
                                     std::mem::drop(file);
@@ -390,9 +381,7 @@ impl Runner {
                                 }
                             }
 
-                            app::MsgOut::CallLuaSilently(func) => {
-                                tx_event_reader.send(true)?;
-
+                            CallLuaSilently(func) => {
                                 match call_lua(&app, &lua, &func, false) {
                                     Ok(Some(msgs)) => {
                                         for msg in msgs {
@@ -407,13 +396,9 @@ impl Runner {
                                         app = app.log_error(err.to_string())?;
                                     }
                                 };
-
-                                tx_event_reader.send(false)?;
                             }
 
-                            app::MsgOut::CallSilently(cmd) => {
-                                tx_event_reader.send(true)?;
-
+                            CallSilently(cmd) => {
                                 app.write_pipes()?;
                                 let status = call(&app, cmd, true)
                                     .map(|s| {
@@ -444,15 +429,13 @@ impl Runner {
                                 if let Err(e) = status {
                                     app = app.log_error(e.to_string())?;
                                 };
-
-                                tx_event_reader.send(false)?;
                             }
 
-                            app::MsgOut::CallLua(func) => {
+                            CallLua(func) => {
                                 execute!(terminal.backend_mut(), event::DisableMouseCapture)
                                     .unwrap_or_default();
 
-                                tx_event_reader.send(true)?;
+                                event_reader::pause_reading(&tx_event_reader, &rx_loop_waiter)?;
 
                                 terminal.clear()?;
                                 terminal.set_cursor(0, 0)?;
@@ -477,7 +460,7 @@ impl Runner {
                                 terminal.clear()?;
                                 term::enable_raw_mode()?;
                                 terminal.hide_cursor()?;
-                                tx_event_reader.send(false)?;
+                                event_reader::resume_reading(&tx_event_reader, &rx_loop_waiter)?;
 
                                 if mouse_enabled {
                                     match execute!(
@@ -494,11 +477,11 @@ impl Runner {
                                 }
                             }
 
-                            app::MsgOut::Call(cmd) => {
+                            Call(cmd) => {
                                 execute!(terminal.backend_mut(), event::DisableMouseCapture)
                                     .unwrap_or_default();
 
-                                tx_event_reader.send(true)?;
+                                event_reader::pause_reading(&tx_event_reader, &rx_loop_waiter)?;
 
                                 terminal.clear()?;
                                 terminal.set_cursor(0, 0)?;
@@ -539,7 +522,7 @@ impl Runner {
                                 terminal.clear()?;
                                 term::enable_raw_mode()?;
                                 terminal.hide_cursor()?;
-                                tx_event_reader.send(false)?;
+                                event_reader::resume_reading(&tx_event_reader, &rx_loop_waiter)?;
 
                                 if mouse_enabled {
                                     match execute!(
